// $Id: Clock.cc,v 1.2.2.1 1998/09/22 07:43:22 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Array.h"
#include "Clock.h"

static void RegisterClock(Clock *clock);
static void UnregisterClock(Clock *clock);


/* Clock */

Clock TheClock;

Clock::Clock() {
	RegisterClock(this);
}

Clock::~Clock() {
	UnregisterClock(this);
}

/* Clock Manager (this could've been an "singular" object) */

static Array<Clock*> *TheClocks = 0;

Time UpdateClocks(bool advanceAll) {
	const Time curTime = Time::Now();
	const Time delay = curTime - TheClock;
	if (advanceAll) {
		for (int i = TheClocks->count()-1; i >=0 ; --i)
			(*TheClocks)[i]->update(curTime);
	} else {
		TheClock.update(curTime);
	}
	return delay;
}

static
void RegisterClock(Clock *clock) {
	TheClocks->append(clock);
}

static
void UnregisterClock(Clock *clock) {
	for (int i = TheClocks->count()-1; i >=0 ; --i) {
		if ((*TheClocks)[i] == clock) {
			(*TheClocks)[i] = (*TheClocks)[TheClocks->count()-1];
			TheClocks->pop(1);
			return;
		}
	}
	abort(); // clock not found
}

/* ClockLibCounter; internal stuff, ignore */

int ClockLibCounter::theUseCount = 0;

ClockLibCounter::ClockLibCounter() {
	if (!theUseCount++)
		TheClocks = new Array<Clock*>;
}

ClockLibCounter::~ClockLibCounter() {
	if (!--theUseCount) {
		assert(TheClocks->count() == 0);
		delete TheClocks;
		TheClocks = 0;
	}
}
