// $Id: Event.cc,v 1.5.2.1 1998/09/22 07:43:26 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Event.h"
#include "Clock.h"
#include "Log.h"
#include "Gadgets.h"

EventQueue TheEventQueue;

EventQueue::~EventQueue() {
	// cancel and delete all pending events
	for (int i = 0; i < theEvents.count(); ++i) {
		theEvents[i]->cancel();
		delete theEvents[i];
	}
}

// execute _all_ pending events
// note: new ones can be coming in the same time
int EventQueue::step() {
	int i = 0;
	Notepad << "<< started event execution with " << theEvents.count() << " events" << ende;
	for (i = 0; i < theEvents.count(); ++i) {
		Event *ev = theEvents[i];
		theEvents[i] = 0;
		noteOut(ev);
		//UpdateClocks(false);
		//const Time execStart = TheClock;
		ev->execute();
		//UpdateClocks(false);
		//Notepad << "-= event[" << theEvents.count() << "] " << *ev << " took: " << (TheClock - execStart) << ende;
		delete ev;
		theExecCount++;
	}
	theEvents.reset();
	Notepad << ">> finished execution of " << theExecCount << " events" << ende;
	theExecCount = 0;
	return i;
}

void EventQueue::noteIn(const Event *ev) const {
	UpdateClocks();
	Notepad << "+= event[" << theEvents.count() << "] "
		<< *ev << ende;
}

void EventQueue::noteOut(const Event *ev) const {
	Notepad << ".= event[" << (theExecCount+1) << "] " 
		<< *ev << ende;
}
