// $Id: LoadGen.cc,v 1.3.2.1 1998/09/22 07:43:30 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"
#include "LoadGen.h"
#include "Log.h"

#include <iostream.h>

static const Time TheWarmUpDelay(10,0); // added to the first pause

/* LoadGenCfg */

LoadGenCfg::LoadGenCfg(): theBlobSize(0), theBlobCount(0),
	theIntraBlobPause(0), theInterBlobPause(0) {
}

void LoadGenCfg::destroy() {
	// XXX: leak: we do not destroy these becuase we do not clone them
	/* delete theIntraBlobPause; */ theIntraBlobPause = 0;
	/* delete theInterBlobPause; */ theInterBlobPause = 0;
}

LoadGenCfg LoadGenCfg::clone() const {
	assert(theIntraBlobPause && theInterBlobPause);
	LoadGenCfg cfg;
	cfg.theBlobSize = theBlobSize;
	cfg.theBlobCount = theBlobCount;
	// XXX: we do not clone these so they would share one "randomness"
	cfg.theIntraBlobPause = theIntraBlobPause; // ->clone();
	cfg.theInterBlobPause = theInterBlobPause; // ->clone();
	return cfg;
}

/* LoadGen */

Loader *LoadGen::clone() const {
	LoadGen *lg = new LoadGen();
	lg->configure(theCfg.clone());
	return lg;
}

void LoadGen::warmUp() {
	sleepFor(TheWarmUpDelay + theCfg.theInterBlobPause->pause());
}

void LoadGen::load() {
	const Time intraPause = theCfg.theIntraBlobPause->pause();
	LoadInfo li = LoadInfo::Delta(
		intraPause > 0 ? +1 : +theCfg.theBlobSize);
	theBlobSize += li.delta();

	changeLoad(li);

	if (blobDone()) {
		theBlobCount++;
		theBlobSize = 0;
		if (enough()) {
			Notepad << "exhausted with " << theBlobCount << " blobs" << ende;
			done();
		} else {
			sleepFor(theCfg.theInterBlobPause->pause());
		}
	} else {
		sleepFor(intraPause);
	}
}


/* Stream Loader */

void StreamLoader::loadChange(const LoadInfo &li) {
	assert(li.level() >= 0);
	if (li.level() < theLevel)
		changeLoad(LoadInfo::Level(theLevel));
}
