// $Id: PushPull.cc,v 1.9.2.1 1998/09/22 07:43:35 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "PushPull.h"
#include "Log.h"

PushPull::~PushPull() {
	if (theConn) {
		Notepad << this << ": releasing lock for " << theKids[0] << " and " << theKids[1] << ende;
		if (theConn->good() && !theConn->exhausted())
			theConn.releaseValid();
		else
			theConn.releaseVoid();
	} else
	if (theConn.queued()) {
		Notepad << this << ": giving up waiting lock for " << theKids[0] << " and " << theKids[1] << ende;
		theConn.giveUpWaiting(this);
	}
	// else we probably was rejected (or deprived from) our resource

	// delete if they still exist
	delete thePusher;
	delete thePuller;
}

void PushPull::obtainedLock(const Mutex &mx) {
	Assert(mx == theConn);
	thePusher->conn(theConn);
	thePuller->conn(theConn);
	if (theRec) {
		thePusher->record(theRec);
		thePuller->record(theRec);
	}
	Notepad << this << ": got lock for " << thePusher << " and " << thePuller << ende;
	beActive(thRunnable);
}

void PushPull::init() {
	Notepad << this << ": asking lock for " << thePusher << " and " << thePuller << ende;
	theConn.obtain(this);
}

void PushPull::run() {
	execKid(thePusher); thePusher = 0;
	execKid(thePuller); thePuller = 0;
	state(thSleeping); // wait for threads to finish
}
