// $Id: RepEater.cc,v 1.13.2.2 1998/09/22 07:43:37 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "RepEater.h"
#include "XactLog.h"
#include "ErrHist.h"
#include "Gadgets.h"

#include "Xaction.h" /* for updating theRespSize */

void RepEater::doContent() {
	assert(theParseState != psDone);
	IOBuffer &buf = theConn->theReaderBuf;
	if (theParseState == psWaitHead) { // waiting for reply header
		if (!theCont.theRepHdr->parse(buf.content()))
			return;
		buf.consumed(theCont.theRepHdr->hdrSize());
		// should we expect reply body?
		if (theCont.theRepHdr->noBody())
			maxOffsetHere();
		else
			theParseState = psWaitCBlock;
	}
	if (theParseState == psWaitCBlock) {
		const Size contOffset = workOffset();
		buf >> theCBlock;
		if (buf.errFmt()) {
			TheErrHist[E_FOREIGN_REP]++;
			SysLog << "foreign reply format near {" << endl
				<< buf.content() << endl << '}' << ende;
			maxOffsetHere(); // will not process this reply further
			buf.err(-1); // will close this connection
			theParseState = psDone;
		} else
		if (theCBlock.contSize() > 0) {
			if (theRec && theCont.theRepHdr->xactId() > 0)
				if (theRec->id() != theCont.theRepHdr->xactId())
					theCont.theRepHdr->xactId(-1); // hits cause old ids
			maxOffset(contOffset + theCBlock.contSize());
			theParseState = psWaitBody;
			// XXX: big kludge
			((Xaction*)parent()->parent())->theRespSize = theCBlock.contSize();
		}
	}
	if (theParseState == psWaitBody) {
		const Size cons_size = min(sizeLeft(), buf.contSize());
		buf.consumed(cons_size);
	}

	if (theParseState != psDone) {
		if (overLimit())
			theParseState = psDone;
		else
		if (buf.eod()) {
			TheErrHist[E_EARLY_REP_END]++;
			SysLog << "unexpected end of reply; state: " << theParseState << " offset " << buf.outOffset() << " need: " << theMaxOffset << " near: " << endl << buf.content() << endl << '}' << ende;
			buf.consumed(buf.contSize()); // will trigger exhausted state
		}
	}
}
