#ifndef _CONN_INFO_H_
#define _CONN_INFO_H_ "$Id: ConnInfo.h,v 1.6.2.1 1998/09/22 07:44:00 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "SafePtr.h"
#include "ConnMutex.h"

// a record used by Server to store information on connections

class ConnInfo {
	public:
		typedef enum { dirNone, dirIn, dirOut } Dir;

	public:
		ConnInfo(ResOwner *owner, Connection *aConn);
		~ConnInfo();

		bool idle() const { return theInMgr->idle() && theOutMgr->idle(); }
		bool good() const { return theConn->good(); }
		bool exhausted() const { return theConn->exhausted(); }
		int useCount() const { return theUseCount; }
		
		int id() const { return theId; }
		int idx() const { return theIdx; }
		void idx(int anIdx) { theIdx = anIdx; theInMgr->resIdx(anIdx); theOutMgr->resIdx(anIdx); }
		
		Connection *conn() { return theConn; }
		ResMgr *resMgr(Dir dir) { return (dir == dirIn) ? theInMgr : (dir == dirOut) ? theOutMgr : 0; }
		ConnMx getMx(Dir dir);

		void use() { theUseCount++; }

	protected:
		Connection *theConn;
		ResMgr *theInMgr;
		ResMgr *theOutMgr;
		int theUseCount;

		int theId;  /* unique connection id */
		int theIdx; /* index in the conns array on the server */

	protected:
		static int TheLastId;
};

#endif
