// $Id: PhaseMgr.h,v 1.1.2.2 1998/10/16 02:20:47 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _PHASE_MGR_H_
#define _PHASE_MGR_H_


#include "RateMeter.h"

#include <iostream.h>

// keeps the values of real meters
class MeterShot {
	public:
		MeterShot() {}

		const RateMeter &subm() const { return theSubmMeter; }
		const RateMeter &compl() const { return theComplMeter; }
		const RateMeter &err() const { return theErrMeter; }
		Time time() const { return theShotTime; }

		void reset();
		void shoot();

		MeterShot &operator -=(const MeterShot &s);

	protected:
		RateMeter theSubmMeter;
		RateMeter theComplMeter;
		RateMeter theErrMeter;
		Time theShotTime;
};

extern MeterShot operator -(const MeterShot &s1, const MeterShot &s2);

// maintains info about one metering phase
class PhaseMeter {
	public:
		PhaseMeter(): theName(""), theGoal(-1) {}

		void name(const char *aName) { theName = aName; }

		bool done() const;
		MeterShot diff() const { return theFinishShot - theStartShot; }

		void goal(int absComplCount) { theGoal = absComplCount; }
		void incGoal(int delta);

		void start() { theStartShot.shoot(); }
		void stop() { theFinishShot.shoot(); }

		void report(ostream &os) const;

	protected:
		const char *theName;
		MeterShot theStartShot;
		MeterShot theFinishShot;
		int theGoal;
};

// maintains phases
class PhaseMgr {
	public:
		PhaseMgr();

		void goal(int aGoal);

		MeterShot mshot() const;

		void checkPhase();

	protected:
		void nextPhase();

	protected:
		PhaseMeter thePhases[3];
		PhaseMeter thePinger;
		int theGoal;           // expected total number of completions
		enum Phase { phNone = -1, phWarmUp = 0, phMeasure, phCoolOff } thePhase;
};

#endif
