#ifndef _PUSH_PULL_H_
#define _PUSH_PULL_H_ "$Id: PushPull.h,v 1.10.2.1 1998/09/22 07:44:16 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Process.h"
#include "XactThread.h"

class XactRec;

// a process that holds a pair of xaction threads working on one IO buffer
class PushPull: public Process, public Mutexer {
	public:
		PushPull(XactThread *aPusher, XactThread *aPuller, XactRec *aRec): thePusher(aPusher), thePuller(aPuller), theRec(aRec) {}
		virtual ~PushPull();

		void conn(ConnMx &aConn) { theConn = aConn; }
		void record(XactRec *aRec) { theRec = aRec; }

		virtual Thread *clone() const { return new PushPull((XactThread*)thePusher->clone(), (XactThread*)thePuller->clone(), 0); }

		virtual void obtainedLock(const Mutex &);
		virtual void deprivedLock(const Mutex &) { theConn = 0; beActive(thDone); } // lost obtained lock
		virtual void refusedLock(const Mutex &) { theConn = 0; beActive(thDone); }  // will never obtain

		virtual void init();
		virtual void run();

		virtual const char *name() const { return "PushPull"; }

	protected:
		ConnMx theConn;
		XactThread *thePusher;
		XactThread *thePuller;
		XactRec *theRec;
};

#endif
